@@ -1,3 +1,26 @@
+'''
+Infer pseudotime and lineage tree from nearest neighbors graph of multiome data.
+Uses an adaptation of the Palantir algorithm to calculate diffusion
+pseudotime and terminal state probabilities for each cell. Then, applies
+MIRA's lineage tree inference algorithm to reconstruct bicurcating tree
+structure of differentation data.
+
+Examples
+--------
+>>> sc.tl.diffmap(adata)
+>>> mira.pl.plot_eigengap(adata)
+>>> sc.pp.neighbors(adata, use_rep = 'X_diffmap', key_added = 'X_diffmap', n_neighbors = 30, n_pcs = 3)
+>>> mira.time.normalize_diffmap(adata)
+>>> mira.get_connected_components(adata)
+>>> mira.time.get_transport_map(adata, start_cell = 0)
+>>> mira.time.find_terminal_cells(adata)
+[10,20,30]
+>>> mira.time.get_branch_probabilities(adata, terminal_cells = {
+    "lineage1" : 10, "lineage2" : 20, "lineage3" : 3
+})
+>>> mira.time.get_tree_structure(adata, treshold = 0.5)
+'''
+
 from networkx.algorithms import components
 import numpy as np
 import networkx as nx
@@ -77,22 +100,52 @@ def get_dendogram_levels(G):
 ### ___ PALANTIR FUNCTIONS ____ ##
 
 @adi.wraps_functional(pti.fetch_diffmap_eigvals, pti.add_diffmap, ['diffmap','eig_vals'])
-def normalize_diffmap(num_comps = None, rescale = True,*,diffmap, eig_vals):
+def normalize_diffmap(rescale = True,*,diffmap, eig_vals):
+    '''
+    Calculates the eigengap heuristic for selecting optimal number of
+    diffusion components to represent dataset. By default,
+    rescales and normalizes the l2 norm of each eigenvector to prioritize
+    components with larger eigenvalues. 
 
-    eigen_gap = eig_vals[:-1] - eig_vals[1:]
-    if num_comps is None:
-        num_comps = np.maximum(np.argmax(eigen_gap), 3) + 1
-    else:
-        assert(isinstance(num_comps, int) and num_comps > 1)
-        num_comps+=1
+    Rescaling eigenvectors produces a smoother pseudotime representation,
+    but may over-smooth the nearest neighbor space and may distort branch
+    points and lineage paths. If branch points appear poorly-defined, try
+    setting `rescale` to False.
 
-    diffmap = diffmap[:, 1:num_comps]
+    Parameters
+    ----------
+    adata : anndata.AnnData
+        Adata with diffusion map calculated.
+    rescale : boolean, default = True
+        Whether to rescale magnitudes of eigenvectors. Rescaling produces smoother pseudotime,
+        but may distort KNN graph and obfuscate fine differences between lineages.
+
+    Returns
+    -------
+    adata : anndata.AnnData
+        `.obs['diffmap']` : np.ndarray[float] of shape (n_cells, n_comps)
+            Rescaled diffusion map subset with estimated 
+            optimal number of components.
+        `.obs['eigen_gap']` : np.ndarray[float] of shape (15,)
+            Eigen gap, difference between previous and current eigenvalues,
+            e.g. [1st - 2nd, 3rd - 2nd, ...]. The component with the
+            largest eigengap is taken to be the number of components needed
+            to represent the dataset, as this represents the point where
+            subsequent eigenvectors drop-off in the amount of variance explained.
+    '''
+
+    eigen_gap = (eig_vals[:-1] - eig_vals[1:])
+    num_comps = np.maximum(np.argmax(eigen_gap), 2)
+
+    diffmap = diffmap[:, 1:]
 
     if rescale:
         diffmap/=np.linalg.norm(diffmap, axis = 0, keepdims = True)
-        eig_vals = eig_vals[1:num_comps]
+        eig_vals = eig_vals[1:]
         diffmap *= (eig_vals / (1 - eig_vals))[np.newaxis, :]
 
+    print('Recommending {} diffusion map components.'.format(str(num_comps)))
+
     return diffmap, eigen_gap
 
 
@@ -137,7 +190,23 @@ def sample_waypoints(num_waypoints = 3000,*, diffmap):
     partial(adi.add_obs_col, colname = 'mira_connected_components'),
     ['connectivities'])
 def get_connected_components(*,connectivities):
+    '''
+    Finds subgraphs in diffusion map or KNN graph. Pseudotime inference
+    algorithms may only be calculated on connected graphs.
 
+    Parameters
+    ----------
+    adata : anndata.AnnData
+    connectivities_key : str, default="X_diffmap_connectivities"
+        Key under `.obsp` to find connectivities of nearest neighbors
+        graph used for pseudotime trajectory inference.
+
+    Returns
+    -------
+    adata : anndata.AnnData
+        `.obs["mira_connected_components"]` : np.ndarray[str] of shape (n_cells,)
+            Label for subgraph membership.
+    '''
     assert(isspmatrix(connectivities))
     N = connectivities.shape[0]
     G = nx.convert_matrix.from_scipy_sparse_matrix(connectivities)
@@ -284,9 +353,45 @@ def get_adaptive_affinity_matrix(*,kernel_width, distance_matrix, pseudotime = N
 
 
 @adi.wraps_functional(pti.fetch_diffmap_distances_and_components, pti.add_transport_map, 
-    ['distance_matrix','diffmap','components'])
-def get_transport_map(ka = 5, n_jobs = -1,*, start_cell, distance_matrix, diffmap,
+    ['distance_matrix','diffmap','components','start_cell'])
+def get_transport_map(ka = 5, n_jobs = 1,*, start_cell = None, distance_matrix, diffmap,
                 components):
+    '''
+    Calculate pseudotime and stochastic forward markov model of differentiation. Each
+    cell is assigned a pseudotime based on its progress through a differentiation. Each
+    cell is also given transition probabilities to other cells within the KNN graph. 
+    Transitions prioritize forward progress to more differentiated states.
+
+    Parameters
+    ----------
+    adata : anndata.AnnData
+        Adata with connected components labeled in `.obs["mira_connected_components"]`
+    ka : int > 5, default = 5
+        Kernel width. The standard deviation of the adaptive gaussian kernel used to
+        convert distances to affinities is taken to be the distance between the current
+        cell and the cell's *ka*th nearest neighbor.
+    n_jobs : int > 0, default = 1
+        Number of cores to use for pseudotime calculation.
+    start_cell : int or barcode
+        Cell representing start state of differentiation.
+    diffmap_distances_key : str, default = "X_diffmap_distances"
+        Key in `.obsp` to find distance matrix between cells. By default, uses
+        distance between cells in diffusion space. Providing "distances" will
+        directly use Joint KNN graph without diffusion smoothing.
+    diffmap_coordinates_key : str, default = "X_diffmap"
+        Key in `.obsm` which holds the coordinates of cells used to calculate
+        the distances in `diffmap_distances_key`.
+
+    Returns
+    -------
+    adata : anndata.AnnData
+        `.obs["mira_pseudotime"]` : np.ndarray[float] of shape (n_cells,)
+            Pseudotime of cells
+        `.obsp["transport_map"]` : scipy.spmatrix[float] of shape (n_cells, n_cells)
+            Sparse matrix of transition probabilities between cells.
+        `.uns["iroot"]` : str
+            name/id of start cell
+    '''
     
     assert(len(np.unique(components)) == 1), 'Graphs with multiple connected components may not be used. Subset cells to include only one connected component.'
 
@@ -307,8 +412,34 @@ def get_transport_map(ka = 5, n_jobs = -1,*, start_cell, distance_matrix, diffma
 
     return pseudotime, transport_map, start_cell
 
-@adi.wraps_functional(pti.fetch_transport_map, adi.return_output, ['transport_map'])
-def find_terminal_cells(iterations = 1, max_termini = 10, threshold = 1e-3, *, transport_map):
+
+@adi.wraps_functional(pti.fetch_transport_map, pti.get_cell_ids, ['transport_map'])
+def find_terminal_cells(iterations = 1, max_termini = 15, threshold = 1e-3, *, transport_map):
+    '''
+    Uses transport map to identify terminal cells where differentiation progress
+    reaches a steady state. Results are stochastic based on SVD initialization.
+
+    Parameters
+    ----------
+    adata : anndata.AnnData
+        Adata with a transition matrix in `.obsp["transport_map"]`
+    iterations : int > 0, default = 1
+        Number of iterations to try eigenvector decomposition of
+        transport map to find terminal cells.
+    max_termini : int > 0, default = 15
+        Maximum number of terminal cells that may be identified. Each
+        possible terminal cell increases computation time.
+    threshold : float > 0, default = 1e-3
+        Terminal cells are taken from eigvectors with eigenvalues equal to 1. 
+        Treshold includes eigenvectors that are greater than 1 - `threshold`.
+        To loosen the definition of terminal states and identify more cells,
+        increase `threshold`, e.g. 1e-2.
+
+    Returns
+    -------
+    terminal_cells : list[str]
+        List of terminal cell names
+    '''
 
     assert(transport_map.shape[0] == transport_map.shape[1])
     assert(len(transport_map.shape) == 2)
@@ -329,8 +460,47 @@ def find_terminal_cells(iterations = 1, max_termini = 10, threshold = 1e-3, *, t
 
     return np.array(list(terminal_points))
 
-@adi.wraps_functional(pti.fetch_transport_map, pti.add_branch_probs, ['transport_map'])
+
+@adi.wraps_functional(pti.fetch_transport_map_and_terminal_cells, pti.add_branch_probs, ['transport_map',
+    'terminal_cells'])
 def get_branch_probabilities(*, transport_map, terminal_cells):
+    '''
+    Simulate forward random walks through transport map modeling stochastic
+    differentiation process. For each cell, calculate probability of 
+    diffusing through map to each terminal state provided by user.
+
+    Parameters
+    ----------
+    adata : anndata.AnnData
+        Adata with a transition matrix in `.obsp["transport_map"]`
+    terminal_cells : dict of format
+        {"lineage_name" : int (cell number), ...} or {"lineage_name" : str (cell barcode), ...}
+        A dictionary mapping lineage names to cells marking terminal states. Cells may
+        be indicated by cell number / index, or by barcode.
+    
+    Returns
+    -------
+    adata : anndata.AnnData
+        `.obsm["branch_probs"]` : np.ndarray[float] of shape (n_cells,n_lineages)
+            Probability of reaching each terminal state following random walk through
+            transport map from each cell.
+        `.obs["differentiation_entropy"]` : np.ndarray[float] of shape (n_cells,)
+            Entropy of differentiation probabilities for each cell.
+        `.uns["lineage_names"]` : list[str]
+            List of lineage names.
+        `.uns["terminal_cells"] : list[str]
+            List of terminal cell barcodes for each lineage.
+        `.obs["<lineage_1>_prob"], ..., .obs["<lineage_L>_prob"]` :  np.ndarray[float] of shape (n_cells,)
+            Column added for the probability of reaching each terminal state for each cell.
+
+    Examples
+    --------
+    >>> mira.time.get_branch_probabilities(adata, terminal_cells = {
+        "A" : "AACACATGTGTAC", "B" : "GTGTGCGAGCGAATT"
+    })
+    >>> sc.pl.umap(adata, color = ["A_prob", "B_prob"])
+    >>> sc.pl.umap(adata, color = 'differentiation_entropy")
+    '''
 
     logger.info('Simulating random walks ...')
 
@@ -370,7 +540,7 @@ def get_branch_probabilities(*, transport_map, terminal_cells):
     branch_probs_including_terminal[absorbing_states_idx, np.arange(num_absorbing_cells)] = 1.
 
     return branch_probs_including_terminal, \
-        list(lineage_names), entropy(branch_probs_including_terminal, axis = -1)
+        list(lineage_names), list(absorbing_states_idx), entropy(branch_probs_including_terminal, axis = -1)
 
 
 ## __ TREE INFERENCE FUNCTIONS __ ##
@@ -410,7 +580,48 @@ def get_lineage_branch_time(lineage1, lineage2, pseudotime, prob_fc, threshold =
 @adi.wraps_functional(pti.fetch_tree_state_args, pti.add_tree_state_args, 
     ['lineage_names', 'branch_probs', 'pseudotime', 'start_cell'])
 def get_tree_structure(threshold = 0.1,*, lineage_names, branch_probs, pseudotime, start_cell):
+    '''
+    Parse tree structure from terminal state probabilities. 
 
+    Parameters
+    ----------
+    adata : anndata.AnnData
+        Adata with a transition matrix in `.obsp["transport_map"]` and
+        branch probabilities in `.obsm["branch_probs"]`.
+    threshold : float > 0, default = 0.1
+        Treshold to exceed to indicate divergence of lineages. Higher thresholds
+        allow more tolerance before a branch point is assigned, while lower 
+        thresholds are more sensitive to divergence events. Try multiple values
+        for threshold to find best value to parse lineage tree. Typically, 0.1 is a
+        very sensitive value, while 1.0-2.0 are very tolerant.
+
+    Returns
+    -------
+        adata : anndata.AnnData
+            `.obs["tree_states"] : np.ndarray[str] of shape (n_cells,)
+                "tree_states" columns indicate which lineages are downstream for
+                each cell. For instance, given a bifurcating differentitation 
+                into terminal lineages A and B, all cells before the branch point
+                would be labeled "A, B", while cells after the branch point would be
+                labeled "B" if they followed the path to the "B" terminal state. 
+            `.uns["connectivities_tree"] : np.ndarray[boolean] of shape (2*num_lineages - 1, 2*num_lineages - 1)
+                Digraph of relationships between `tree_states`. For instance,
+                an edge would be drawn from "A, B" to "A" and to "B".
+
+                                     "A"
+                                    /   
+                              "A, B"
+                                    \__"B"
+
+            `.uns["tree_state_names"] : np.ndarray[str] of shape (2*num_lineages - 1,)
+                Tree state labels for columns and rows of `connectivities_tree`
+
+    Examples
+    --------
+    >>> mira.time.get_tree_structure(adata, threshold = 0.5)
+    >>> sc.pl.umap(adata, color= "tree_states")
+
+    '''
     def get_all_leaves_from_node(edge):
 
         if isinstance(edge, tuple):
@@ -427,6 +638,7 @@ def get_tree_structure(threshold = 0.1,*, lineage_names, branch_probs, pseudotim
             x[:, ~np.isin(np.arange(x.shape[-1]), [col1, col2])]
         ])
 
+    assert(isinstance(threshold, (int, float)) and threshold > 0), '`threshold` must be a float greater than 0'
 
     num_cells, num_lineages = branch_probs.shape
     all_merged = False
