@@ -1,4 +1,5 @@
 from functools import partial
+from os import link
 from scipy import interpolate
 import pyro
 import pyro.distributions as dist
@@ -20,6 +21,8 @@ import mira.adata_interface.core as adi
 import mira.adata_interface.topic_model as tmi
 import gc
 import matplotlib.pyplot as plt
+from scipy.cluster.hierarchy import linkage, to_tree
+import typing as tp
 logger = logging.getLogger(__name__)
 
 class EarlyStopping:
@@ -95,6 +98,28 @@ def get_fc_stack(layer_dims = [256, 128, 128, 128], dropout = 0.2, skip_nonlin =
 
 
 class BaseModel(torch.nn.Module, BaseEstimator):
+    '''
+    Learns regulatory "topics" from single-cell multiomics data. Topics capture 
+    patterns of covariance between gene or cis-regulatory elements. 
+    Each cell is represented by a composition over topics, and each 
+    topic corresponds with activations of co-regulated elements.
+
+    One may use enrichment analysis of the topics to understand signaling 
+    and transcription factor drivers of cell states, and embedding of 
+    cell-topic distributions to visualize and cluster cells, 
+    and to perform pseudotime trajectory inference.
+
+    Examples
+    --------
+    >>> rna_model = mira.topics.ExpressionTopicModel(
+                exogenous_key = 'predict_expression', 
+                endogenous_key = 'highly_variable',
+                counts_layer = 'rawcounts',
+                num_topics = 15,
+            )
+    >>> rna_model.fit(adata)
+    >>> rna_model.predict(adata)
+    '''
 
     I = 50
 
@@ -165,20 +190,11 @@ class BaseModel(torch.nn.Module, BaseEstimator):
             initial_pseudocounts = 50,
             nb_parameterize_logspace = True,
             embedding_size = None,
+            kl_strategy = 'monotonic',
             ):
         '''
         Initialize a new MIRA topic model.
 
-        Learns regulatory "topics" from single-cell multiomics data. Topics capture 
-        patterns of covariance between gene or cis-regulatory elements. 
-        Each cell is represented by a composition over topics, and each 
-        topic corresponds with activations of co-regulated elements.
-
-        One may use enrichment analysis of the topics to understand signaling 
-        and transcription factor drivers of cell states, and embedding of 
-        cell-topic distributions to visualize and cluster cells, 
-        and to perform pseudotime trajectory inference.
-
         Parameters
         ----------
         endogenous_key : str, default=None
@@ -227,6 +243,12 @@ class BaseModel(torch.nn.Module, BaseEstimator):
         nb_parameterize_logspace : boolean, default=True
             Parameterize negative-binomial distribution using log-space probability 
             estimates of gene expression. Is more numerically stable.
+        embedding_size : int > 0 or None, default=None
+            Number of nodes in first encoder neural network layer. Default of *None*
+            gives an embedding size of `hidden`.
+        kl_strategy : {'monotonic','cyclic'}, default='monotonic'
+            Whether to anneal KL term using monotonic or cyclic strategies. Cyclic
+            may produce slightly better models.
 
         Examples
         --------
@@ -258,6 +280,10 @@ class BaseModel(torch.nn.Module, BaseEstimator):
             Results from enrichment analysis of topics. For expression topic model,
             this gives geneset enrichments from Enrichr. For accessibility topic
             model, this gives motif enrichments.
+        topic_cols : list
+            The names of the columns for the topics added by the
+            `predict` method to an anndata object. Useful for quickly accessing
+            topic columns for plotting.
         '''
         super().__init__()
 
@@ -279,6 +305,7 @@ class BaseModel(torch.nn.Module, BaseEstimator):
         self.initial_pseudocounts = initial_pseudocounts
         self.nb_parameterize_logspace = nb_parameterize_logspace
         self.embedding_size = embedding_size
+        self.kl_strategy = kl_strategy
 
     def _set_seeds(self):
         if self.seed is None:
@@ -399,10 +426,19 @@ class BaseModel(torch.nn.Module, BaseEstimator):
 
     @staticmethod
     def _iterate_batch_idx(N, batch_size, bar = False, desc = None):
-
+        
         num_batches = N//batch_size + int(N % batch_size > 0)
+
         for i in range(num_batches) if not bar else tqdm(range(num_batches), desc = desc):
-            yield i * batch_size, (i + 1) * batch_size
+
+            start, end = i * batch_size, (i + 1) * batch_size
+
+            if N - end == 1:
+                yield start, end + 1
+                raise StopIteration()
+            else:
+                yield start, end
+
 
     def _preprocess_endog(self, X):
         raise NotImplementedError()
@@ -433,10 +469,23 @@ class BaseModel(torch.nn.Module, BaseEstimator):
             'cycle_momentum' : False, 'three_phase' : False, 'verbose' : False})
 
     @staticmethod
-    def _get_KL_anneal_factor(step_num, *, n_epochs, n_batches_per_epoch):
-        total_steps = n_epochs * n_batches_per_epoch
+    def _get_monotonic_kl_factor(step_num, *, n_epochs, n_batches_per_epoch):
         
-        return min(1., (step_num + 1)/(total_steps * 1/2 + 1))
+        total_steps = n_epochs * n_batches_per_epoch
+        return min(1., (step_num + 1)/(total_steps * 1/2 + 1)) 
+
+    @staticmethod
+    def _get_cyclic_KL_factor(step_num, *, n_epochs, n_batches_per_epoch):
+        
+        total_steps = n_epochs * n_batches_per_epoch
+        n_cycles = 3
+        tau = ((step_num+1) % (total_steps/n_cycles))/(total_steps/n_cycles)
+
+        if tau > 0.5 or step_num > total_steps:
+            return 1.
+        else:
+            return max(tau/0.5, n_cycles/total_steps)
+
 
     @property
     def highly_variable(self):
@@ -489,8 +538,8 @@ class BaseModel(torch.nn.Module, BaseEstimator):
         Use the learning rate range test (LRRT) to determine minimum and maximum learning
         rates that enable the model to traverse the gradient of the loss. 
 
-        Steps through linear increase in log-learning rate from **lower_bound_lr** 
-        to **upper_bound_lr** while recording loss of model. Learning rates which
+        Steps through linear increase in log-learning rate from `lower_bound_lr`
+        to `upper_bound_lr` while recording loss of model. Learning rates which
         produce greater decreases in model loss mark range of possible
         learning rates.
 
@@ -499,7 +548,7 @@ class BaseModel(torch.nn.Module, BaseEstimator):
         adata : anndata.AnnData
             AnnData of expression or accessibility features to model
         num_epochs : int, default=6
-            Number of epochs to run the LRRT, may be decreased for smaller datasets.
+            Number of epochs to run the LRRT, may be decreased for larger datasets.
         eval_every : int, default=10,
             Aggregate this number of batches per evaluation of the objective loss.
             Larger numbers give lower variance estimates of model performance.
@@ -548,7 +597,7 @@ class BaseModel(torch.nn.Module, BaseEstimator):
             'lr_lambda' : lr_function})
 
         self.svi = SVI(self.model, self.guide, scheduler, loss=TraceMeanField_ELBO())
-        batches_complete, steps_complete, step_loss = 0,0,0
+        batches_complete, steps_complete, step_loss, samples_seen = 0,0,0,0
         learning_rate_losses = []
         
         try:
@@ -565,12 +614,13 @@ class BaseModel(torch.nn.Module, BaseEstimator):
 
                     step_loss += float(self.svi.step(**batch, anneal_factor = 1.))
                     batches_complete+=1
+                    samples_seen += batch['endog_features'].shape[0]
                     
                     if batches_complete % eval_every == 0 and batches_complete > 0:
                         steps_complete+=1
                         scheduler.step()
-                        learning_rate_losses.append(step_loss/(eval_every * self.batch_size * self.num_exog_features))
-                        step_loss = 0.0
+                        learning_rate_losses.append(step_loss/(samples_seen * self.num_exog_features))
+                        step_loss, samples_seen = 0.0, 0
                         try:
                             next(_t)
                         except StopIteration:
@@ -645,7 +695,7 @@ class BaseModel(torch.nn.Module, BaseEstimator):
         lower_bound_trim : float>=0, default=0.
             Log increase in learning rate of lower bound relative to estimated
             boundary given from LRRT. For example, if the estimated boundary by
-            LRRT is 1e-4 and user gives **lower_bound_trim**=1, the new lower
+            LRRT is 1e-4 and user gives `lower_bound_trim`=1, the new lower
             learning rate bound is set at 1e-3.
         upper_bound_trim : float>=0, default=0.5,
             Log decrease in learning rate of upper bound relative to estimated
@@ -733,7 +783,7 @@ class BaseModel(torch.nn.Module, BaseEstimator):
     def instantiate_model(self,*, features, highly_variable, endog_features, exog_features):
         '''
         Given the exogenous and enxogenous features provided, instantiate weights
-        of neural network. Called internally by **fit**.
+        of neural network. Called internally by `fit`.
 
         Parameters
         ----------
@@ -773,7 +823,7 @@ class BaseModel(torch.nn.Module, BaseEstimator):
 
         self.training_loss, self.testing_loss, self.num_epochs_trained = [],[],0
         
-        anneal_fn = partial(self._get_KL_anneal_factor, n_epochs = self.num_epochs, 
+        anneal_fn = partial(self._get_cyclic_KL_factor if self.kl_strategy == 'cyclic' else self._get_monotonic_kl_factor, n_epochs = self.num_epochs, 
             n_batches_per_epoch = n_batches)
 
         step_count = 0
@@ -906,12 +956,16 @@ class BaseModel(torch.nn.Module, BaseEstimator):
 
     @staticmethod
     def centered_boxcox_transform(x, a = 'log'):
-        if not (a == 'log' or a == 0):
-            x = (x**a - 1)/a
-        else:
-            x = np.log(x)
 
-        return x - x.mean(-1, keepdims = True)
+        if a == 'log' or a == 0:
+            return np.log(x) - np.log(x).mean(-1, keepdims = True)
+
+        else:
+
+            assert(isinstance(a, float) and a > 0 and a < 1)
+            x = (x**a)/(x**a).mean(-1, keepdims = True)
+            return ( x - 1 )/a
+
 
     @staticmethod
     def _gram_schmidt_basis(n):
@@ -927,10 +981,75 @@ class BaseModel(torch.nn.Module, BaseEstimator):
 
         basis = self._gram_schmidt_basis(compositions.shape[-1])
         return self.centered_boxcox_transform(compositions, a = a).dot(basis)
+
+    @adi.wraps_modelfunc(tmi.fetch_topic_comps, tmi.add_phylo,
+        fill_kwargs=['topic_compositions'])
+    def get_hierarchical_umap_features(self, box_cox = 0.5,*, topic_compositions):
+        '''
+        Leverage the hiearchical relationship between topic-feature distributions
+        to prduce a balance matrix for isometric logratio transformation. The 
+        function `get_umap_features` uses an arbitrary balance matrix that 
+        does not account for the relationship between topics.
+
+        Parameters
+        ----------
+        adata : anndata.AnnData
+            AnnData of expression or accessibility features to model
+        box_cox : "log" or float between 0 and 1
+            Constant for box-cox transformation of topic compositions
+
+        Returns
+        -------
+        adata : anndata.AnnData
+            `.obsm['X_umap_features']` : np.ndarray[float] of shape (n_cells, n_topics)
+                Transformed topic compositions of cells
+            `.uns['topic_dendogram']` : np.ndarray
+                linkage matrix given by scipy.cluster.hierarchy.linkage of
+                hiearchical relationship between topic-feature distributions.
+                Hierarchy calculated by hellinger distance and ward linkage.
+        '''
+
+        topics = np.abs(self._get_gamma())[np.newaxis, :] * self._score_features()[:self.num_topics, :] + self._get_bias()[np.newaxis, :]
+        topics = np.sqrt(np.exp(topics)/np.exp(topics).sum(-1, keepdims = True))
+        linkage_matrix = linkage(topics, method='ward', metric='euclidean')
+        
+        root, nodes = to_tree(linkage_matrix, rd = True)
+
+        def children(x):
+
+            if x is None:
+                return []
+
+            if x.left is None and x.right is None:
+                return [x.id]
+
+            return children(x.left) + children(x.right)
+        
+        balance_matrix = np.zeros((self.num_topics -1, self.num_topics))
+        
+        n_plus, n_minus = [],[]
+        for i, node in enumerate(nodes[self.num_topics:]):
+            balance_matrix[i, children(node.left)] = 1
+            balance_matrix[i, children(node.right)] = -1
+            n_plus.append(len(children(node.left)))
+            n_minus.append(len(children(node.right)))
+
+        n_plus, n_minus = np.array(n_plus), np.array(n_minus)
+
+        plus_sites = 1/n_plus * np.sqrt((n_plus*n_minus)/(n_plus + n_minus))
+        minus_sites = -1/n_minus * np.sqrt((n_plus*n_minus)/(n_plus + n_minus))
+
+        g_matrix = np.zeros_like(balance_matrix) + (balance_matrix == 1).astype(int) * plus_sites[:,np.newaxis] +\
+            (balance_matrix == -1).astype(int) * minus_sites[:,np.newaxis]
+
+        umap_features = self.centered_boxcox_transform(topic_compositions, a = box_cox).dot(g_matrix.T)
+
+        return umap_features, linkage_matrix
+
     
     @adi.wraps_modelfunc(tmi.fetch_topic_comps, partial(adi.add_obsm, add_key = 'X_umap_features'),
         fill_kwargs=['topic_compositions'])
-    def get_umap_features(self, box_cox = 'log', *, topic_compositions):
+    def get_umap_features(self, box_cox = 0.5, *, topic_compositions):
         '''
         Predict transformed topic compositions for each cell to derive nearest-
         neighbors graph. Projects topic compositions to orthonormal space using
@@ -940,7 +1059,7 @@ class BaseModel(torch.nn.Module, BaseEstimator):
         ----------
         adata : anndata.AnnData
             AnnData of expression or accessibility features to model
-        box_cox : "log" or int > 0
+        box_cox : "log" or float between 0 and 1
             Constant for box-cox transformation of topic compositions
 
         Returns
@@ -1023,7 +1142,7 @@ class BaseModel(torch.nn.Module, BaseEstimator):
     def impute(self, batch_size = 512, bar = True, *, topic_compositions):
         '''
         Impute the relative frequencies of features given the cells' topic
-        compositions. The value given is **rho** (see manscript for details).
+        compositions. The value given is *rho* (see manscript for details).
 
         Parameters
         ----------
@@ -1148,4 +1267,18 @@ class BaseModel(torch.nn.Module, BaseEstimator):
 
     @property
     def topic_cols(self):
+        '''
+        Attribute, returns the names of the columns for the topics added by the
+        `predict` method to an anndata object. Useful for quickly accessing
+        topic columns for plotting.
+
+        Examples
+        --------
+        >>> model.num_topics
+        5
+        >>> model.topic_cols
+        ['topic_0', 'topic_1','topic_2','topic_3','topic_4']
+        >>> sc.pl.umap(adata, color = model.topic_cols, **mira.pref.topic_umap())
+
+        '''
         return ['topic_' + str(i) for i in range(self.num_topics)]
\ No newline at end of file
